# ErrorlessRebuild
#
# Release 1.00 (Nov 27, 2012)
#
# Copyright (c) 2012 Yujiro Araki

package ErrorlessRebuild::App;

use strict;

use base qw( MT::App );
use MT::Util qw( encode_html encode_url epoch2ts format_ts remove_html relative_date start_end_week week2ymd );
use MT::I18N qw( first_n_text );
use MT::PublishOption;
use ErrorlessRebuild::CMS qw(rest_day is_limit);

use Data::Dumper;

sub init {
    my $app = shift;
    $app->SUPER::init(@_);
}

sub init_request {
    my $app = shift;
    $app->SUPER::init_request(@_);
    $app->add_methods( get_rebuild_data => \&get_rebuild_data );

    return if is_limit();

    $app->{ default_mode } = 'get_rebuild_data';
    $app->{ requires_login } = 0;
}

sub get_rebuild_data {
    my $app = shift;
    my $q = $app->param;

    return if is_limit();

    my $blog_id = $q->param('blog_id') if $q->param('blog_id');
    my @types = split(/,/, $q->param('type')) if $q->param('type');

    my $plugin = MT->component('ErrorlessRebuild');
    my $class = $app->model( 'template' ) or return $app->errtrans( 'Unsupported type' );

    my @entry_ids;
    my @page_ids;
    my @year_times;
    my @month_times;
    my @week_times;
    my @day_times;
    my @cats;
    my @ycats;
    my @mcats;
    my @wcats;
    my @dcats;
    my @authors;
    my @yauthors;
    my @mauthors;
    my @wauthors;
    my @dauthors;
    my @template_ids;
    my @all;

    my $data2;

    my @archives;
    my $blog = MT::Blog->load( $blog_id );

    my %param;

    foreach my $type (@types) {

        # entry
        if ( $type eq 'Individual' ) {
            if ( MT::PublishOption::archive_build_type( $blog_id, 'Individual' ) != MT::PublishOption::DISABLED()) {

            my $iter = MT::Entry->load_iter( { blog_id => $blog_id, class => 'entry', status => MT::Entry::RELEASE() } );
            while ( my $entry = $iter->() ) {
                push @entry_ids, $blog_id . ':' . $entry->id;
                @entry_ids = sort { $b cmp $a } @entry_ids;
            }
            }

        }

        # page
        if ( $type eq 'Page' ) {
            if ( MT::PublishOption::archive_build_type( $blog_id, 'Page' ) != MT::PublishOption::DISABLED()) {
            my $iter = MT::Entry->load_iter( { blog_id => $blog_id, class => 'page', status => MT::Entry::RELEASE() } );
            while ( my $page = $iter->() ) {
                push @page_ids, $blog_id . ':' . $page->id;
                @page_ids = sort { $b cmp $a } @page_ids;
            }
            }
        }

        # yearly
        if ( $type eq 'Yearly' ) {
            if ( MT::PublishOption::archive_build_type( $blog_id, 'Yearly' ) != MT::PublishOption::DISABLED()) {
            my $year_timestamp;
            my $yearly_list;
            my $iter = MT::Entry->load_iter( { blog_id => $blog_id } );
            while ( my $entry = $iter->() ) {
                ( $year_timestamp = $entry->authored_on ) =~ s/^(\d{4})\d{10}$/${1}0000000000/;
                $yearly_list->{ $year_timestamp } = 1;
            }
            for my $key ( keys %$yearly_list ) {
                push @year_times, $blog_id . ':' . $key;
                @year_times = sort { $b cmp $a } @year_times;
            }
            }
        }

        # monthly
        if ( $type eq 'Monthly' ) {
            if ( MT::PublishOption::archive_build_type( $blog_id, 'Monthly' ) != MT::PublishOption::DISABLED()) {
            my $month_timestamp;
            my $monthly_list;
            my $iter = MT::Entry->load_iter( { blog_id => $blog_id } );
            while ( my $entry = $iter->() ) {
                ( $month_timestamp = $entry->authored_on ) =~ s/^(\d{4})(\d{2})\d{8}$/$1${2}00000000/;
                $monthly_list->{ $month_timestamp } = 1;
            }
            for my $key ( keys %$monthly_list ) {
                push @month_times, $blog_id . ':' . $key;
                @month_times = sort { $b cmp $a } @month_times;
            }
            }
        }

        # weekly
        if ( $type eq 'Weekly' ) {
            if ( MT::PublishOption::archive_build_type( $blog_id, 'Weekly' ) != MT::PublishOption::DISABLED()) {
            my $week_timestamp;
            my $weekly_list;
            my $iter = MT::Entry->load_iter( { blog_id => $blog_id } );
            while ( my $entry = $iter->() ) {
                my ( $start, $end ) = start_end_week( $entry->authored_on );
                $weekly_list->{ $start } = 1;
            }
            for my $key ( keys %$weekly_list ) {
                push @week_times, $blog_id . ':' . $key;
                @week_times = sort { $b cmp $a } @week_times;
            }
            }
        }

        # daily
        if ( $type eq 'Daily' ) {
            if ( MT::PublishOption::archive_build_type( $blog_id, 'Daily' ) != MT::PublishOption::DISABLED()) {
            my $day_timestamp;
            my $daily_list;
            my $iter = MT::Entry->load_iter( { blog_id => $blog_id } );
            while ( my $entry = $iter->() ) {
                ( $day_timestamp = $entry->authored_on ) =~ s/^(\d{4})(\d{2})(\d{2})\d{6}$/$1$2${3}000000/;
                $daily_list->{ $day_timestamp } = 1;
            }
            for my $key ( keys %$daily_list ) {
                push @day_times, $blog_id . ':' . $key;
                @day_times = sort { $b cmp $a } @day_times;
            }
            }
        }

        # category
        if ( $type eq 'Category' ) {
            if ( MT::PublishOption::archive_build_type( $blog_id, 'Category' ) != MT::PublishOption::DISABLED()) {
            require MT::TemplateMap;
            my $iter = MT::Category->load_iter( { blog_id => $blog_id, class => 'category' } );
            while ( my $cat = $iter->() ) {
                push @cats, $blog_id . ':' . $cat->id;
            }
            }
        }

        # category-yearly
        if ( $type eq 'Category-Yearly' ) {
            if ( MT::PublishOption::archive_build_type( $blog_id, 'Category-Yearly' ) != MT::PublishOption::DISABLED()) {
            my $year_timestamp;
            my $yearly_list;
            my @year_times;

            my $iter = MT::Category->load_iter( { blog_id => $blog_id, class => 'category' } );
            while ( my $cat = $iter->() ) {
                my ( $terms, $args );
                my ( $join_terms, $join_args );
                $join_terms->{ category_id } = $cat->id;
                $join_args->{ unique } = 1;
                $terms->{ blog_id } = $blog_id;
                $terms->{ status } = MT::Entry::RELEASE();
                $args->{ join } = MT::Placement->join_on( 'entry_id', $join_terms );
                $args->{ sort } = 'authored_on';
                $args->{ direction } = 'descend';
                my $iter2 = MT::Entry->load_iter( $terms, $args );

                undef $yearly_list;
                while ( my $entry = $iter2->() ) {
                    ( $year_timestamp = $entry->authored_on ) =~ s/^(\d{4})\d{10}$/${1}0000000000/;
                    $yearly_list->{ $year_timestamp } = 1;
                }
                @year_times = ();
                for my $key ( keys %$yearly_list ) {
                    push @year_times, $key;
                }

                foreach my $time ( @year_times ) {
                    push @ycats, $blog_id . ':' . $cat->id . "#" . $time;
                }
                @ycats = sort { $b cmp $a } @ycats;
            }
            }
        }

        # category-monthly
        if ( $type eq 'Category-Monthly' ) {
            if ( MT::PublishOption::archive_build_type( $blog_id, 'Category-Monthly' ) != MT::PublishOption::DISABLED()) {
            my $month_timestamp;
            my $monthly_list;
            my @month_times;

            my $iter = MT::Category->load_iter( { blog_id => $blog_id, class => 'category' } );
            while ( my $cat = $iter->() ) {
                my ( $terms, $args );
                my ( $join_terms, $join_args );
                $join_terms->{ category_id } = $cat->id;
                $join_args->{ unique } = 1;
                $terms->{ status } = MT::Entry::RELEASE();
                $terms->{ blog_id } = $blog_id;
                $args->{ join } = MT::Placement->join_on( 'entry_id', $join_terms );
                $args->{ sort } = 'authored_on';
                $args->{ direction } = 'descend';
                my $iter2 = MT::Entry->load_iter( $terms, $args );

                undef $monthly_list;
                while ( my $entry = $iter2->() ) {
                    ( $month_timestamp = $entry->authored_on ) =~ s/^(\d{4})(\d{2})\d{8}$/$1${2}00000000/;
                    $monthly_list->{ $month_timestamp } = 1;
                }
                @month_times = ();
                for my $key ( keys %$monthly_list ) {
                    push @month_times, $key;
                }

                foreach my $time ( @month_times ) {
                    push @mcats, $blog_id . ':' . $cat->id . "#" . $time;
                }
                @mcats = sort { $b cmp $a } @mcats;
            }
            }
        }

        # category-weekly
        if ( $type eq 'Category-Weekly' ) {
            if ( MT::PublishOption::archive_build_type( $blog_id, 'Category-Weekly' ) != MT::PublishOption::DISABLED()) {
            my $week_timestamp;
            my $weekly_list;
            my @week_times;

            my $iter = MT::Category->load_iter( { blog_id => $blog_id, class => 'category' } );
            while ( my $cat = $iter->() ) {
                my ( $terms, $args );
                my ( $join_terms, $join_args );
                $join_terms->{ category_id } = $cat->id;
                $join_args->{ unique } = 1;
                $terms->{ status } = MT::Entry::RELEASE();
                $terms->{ blog_id } = $blog_id;
                $args->{ join } = MT::Placement->join_on( 'entry_id', $join_terms );
                $args->{ sort } = 'authored_on';
                $args->{ direction } = 'descend';
                my $iter2 = MT::Entry->load_iter( $terms, $args );

                undef $weekly_list;
                while ( my $entry = $iter2->() ) {
                    my ( $start, $end ) = start_end_week( $entry->authored_on );
                    $weekly_list->{ $start } = 1;
                }
                @week_times = ();
                for my $key ( keys %$weekly_list ) {
                    push @week_times, $key;
                }

                foreach my $time ( @week_times ) {
                    push @wcats, $blog_id . ':' . $cat->id . "#" . $time;
                }
                @wcats = sort { $b cmp $a } @wcats;
            }
            }
        }

        # category-daily
        if ( $type eq 'Category-Daily' ) {
            if ( MT::PublishOption::archive_build_type( $blog_id, 'Category-Daily' ) != MT::PublishOption::DISABLED()) {
            my $day_timestamp;
            my $daily_list;
            my @day_times;

            my $iter = MT::Category->load_iter( { blog_id => $blog_id, class => 'category' } );
            while ( my $cat = $iter->() ) {
                my ( $terms, $args );
                my ( $join_terms, $join_args );
                $join_terms->{ category_id } = $cat->id;
                $join_args->{ unique } = 1;
                $terms->{ status } = MT::Entry::RELEASE();
                $terms->{ blog_id } = $blog_id;
                $args->{ join } = MT::Placement->join_on( 'entry_id', $join_terms );
                $args->{ sort } = 'authored_on';
                $args->{ direction } = 'descend';
                my $iter2 = MT::Entry->load_iter( $terms, $args );

                undef $daily_list;
                while ( my $entry = $iter2->() ) {
                    ( $day_timestamp = $entry->authored_on ) =~ s/^(\d{4})(\d{2})(\d{2})\d{6}$/$1$2${3}000000/;
                    $daily_list->{ $day_timestamp } = 1;
                }
                @day_times = ();
                for my $key ( keys %$daily_list ) {
                    push @day_times, $key;
                }
            
                foreach my $time ( @day_times ) {
                    push @dcats, $blog_id . ':' . $cat->id . "#" . $time;
                }
                @dcats = sort { $b cmp $a } @dcats;
            }
            }
        }

        # author
        if ( $type eq 'Author' ) {
            if ( MT::PublishOption::archive_build_type( $blog_id, 'Author' ) != MT::PublishOption::DISABLED()) {

            my ( $args, $terms );
            $args->{sort} = 'id';
            $args->{unique} = 1;
            $terms->{status} = MT::Author::ACTIVE();
            require MT::Entry;
            $args->{join} = MT::Entry->join_on(
                'author_id',
                {   blog_id => $blog->id,
                    class   => 'entry',
                    status  => MT::Entry::RELEASE()
                },
                { unique => 1 }
            );
            my $iter = MT::Author->load_iter( $terms, $args );
            while ( my $author = $iter->() ) {
                push @authors, $blog_id . ':' . $author->id;
            }

            }
        }

        # author-yearly
        if ( $type eq 'Author-Yearly' ) {
            if ( MT::PublishOption::archive_build_type( $blog_id, 'Author-Yearly' ) != MT::PublishOption::DISABLED()) {

            my $year_timestamp;
            my $yearly_list;
            my @year_times;

            my ( $args, $terms );
            $args->{sort} = 'id';
            $args->{unique} = 1;
            $terms->{status} = MT::Author::ACTIVE();
            require MT::Entry;
            $args->{join} = MT::Entry->join_on(
                'author_id',
                {   blog_id => $blog->id,
                    class   => 'entry',
                    status  => MT::Entry::RELEASE()
                },
                { unique => 1 }
            );
            my $iter = MT::Author->load_iter( $terms, $args );
            while ( my $author = $iter->() ) {
                my ( $terms, $args );
                $terms->{ status } = MT::Entry::RELEASE();
                $terms->{ blog_id } = $blog_id;
                $terms->{ author_id } = $author->id;
                $args->{ sort } = 'authored_on';
                $args->{ direction } = 'descend';
                my $iter2 = MT::Entry->load_iter( $terms, $args );

                undef $yearly_list;
                while ( my $entry = $iter2->() ) {
                    ( $year_timestamp = $entry->authored_on ) =~ s/^(\d{4})\d{10}$/${1}0000000000/;
                    $yearly_list->{ $year_timestamp } = 1;
                }
                @year_times = ();
                for my $key ( keys %$yearly_list ) {
                    push @year_times, $key;
                }

                foreach my $time ( @year_times ) {
                    push @yauthors, $blog_id . ':' . $author->id . "#" . $time;
                }
                @yauthors = sort { $b cmp $a } @yauthors;
            }
            }
        }

        # author-monthly
        if ( $type eq 'Author-Monthly' ) {
            if ( MT::PublishOption::archive_build_type( $blog_id, 'Author-Monthly' ) != MT::PublishOption::DISABLED()) {

            my $month_timestamp;
            my $monthly_list;
            my @month_times;

            my ( $args, $terms );
            $args->{sort} = 'id';
            $args->{unique} = 1;
            $terms->{status} = MT::Author::ACTIVE();
            require MT::Entry;
            $args->{join} = MT::Entry->join_on(
                'author_id',
                {   blog_id => $blog->id,
                    class   => 'entry',
                    status  => MT::Entry::RELEASE()
                },
                { unique => 1 }
            );
            my $iter = MT::Author->load_iter( $terms, $args );
            while ( my $author = $iter->() ) {
                my ( $terms, $args );
                $terms->{ status } = MT::Entry::RELEASE();
                $terms->{ blog_id } = $blog_id;
                $terms->{ author_id } = $author->id;
                $args->{ sort } = 'authored_on';
                $args->{ direction } = 'descend';
                my $iter2 = MT::Entry->load_iter( $terms, $args );

                undef $monthly_list;
                while ( my $entry = $iter2->() ) {
                    ( $month_timestamp = $entry->authored_on ) =~ s/^(\d{4})(\d{2})\d{8}$/$1${2}00000000/;
                    $monthly_list->{ $month_timestamp } = 1;
                }
                @month_times = ();
                for my $key ( keys %$monthly_list ) {
                    push @month_times, $key;
                }

                foreach my $time ( @month_times ) {
                    push @mauthors, $blog_id . ':' . $author->id . "#" . $time;
                }
                @mauthors = sort { $b cmp $a } @mauthors;
            }
            }
        }

        # author-weekly
        if ( $type eq 'Author-Weekly' ) {
            if ( MT::PublishOption::archive_build_type( $blog_id, 'Author-Weekly' ) != MT::PublishOption::DISABLED()) {
            my $week_timestamp;
            my $weekly_list;
            my @week_times;

            my ( $args, $terms );
            $args->{sort} = 'id';
            $args->{unique} = 1;
            $terms->{status} = MT::Author::ACTIVE();
            require MT::Entry;
            $args->{join} = MT::Entry->join_on(
                'author_id',
                {   blog_id => $blog->id,
                    class   => 'entry',
                    status  => MT::Entry::RELEASE()
                },
                { unique => 1 }
            );
            my $iter = MT::Author->load_iter( $terms, $args );
            while ( my $author = $iter->() ) {
                my ( $terms, $args );
                $terms->{ status } = MT::Entry::RELEASE();
                $terms->{ blog_id } = $blog_id;
                $terms->{ author_id } = $author->id;
                $args->{ sort } = 'authored_on';
                $args->{ direction } = 'descend';
                my $iter2 = MT::Entry->load_iter( $terms, $args );

                undef $weekly_list;
                while ( my $entry = $iter2->() ) {
                    my ( $start, $end ) = start_end_week( $entry->authored_on );
                    $weekly_list->{ $start } = 1;
                }
                @week_times = ();
                for my $key ( keys %$weekly_list ) {
                    push @week_times, $key;
                }

                foreach my $time ( @week_times ) {
                    push @wauthors, $blog_id . ':' . $author->id . "#" . $time;
                }
                @wauthors = sort { $b cmp $a } @wauthors;
            }
            }
        }

        # author-daily
        if ( $type eq 'Author-Daily' ) {
            if ( MT::PublishOption::archive_build_type( $blog_id, 'Author-Daily' ) != MT::PublishOption::DISABLED()) {
            my $day_timestamp;
            my $daily_list;
            my @day_times;

            my ( $args, $terms );
            $args->{sort} = 'id';
            $args->{unique} = 1;
            $terms->{status} = MT::Author::ACTIVE();
            require MT::Entry;
            $args->{join} = MT::Entry->join_on(
                'author_id',
                {   blog_id => $blog->id,
                    class   => 'entry',
                    status  => MT::Entry::RELEASE()
                },
                { unique => 1 }
            );
            my $iter = MT::Author->load_iter( $terms, $args );
            while ( my $author = $iter->() ) {
                my ( $terms, $args );
                $terms->{ status } = MT::Entry::RELEASE();
                $terms->{ blog_id } = $blog_id;
                $terms->{ author_id } = $author->id;
                $args->{ sort } = 'authored_on';
                $args->{ direction } = 'descend';
                my $iter2 = MT::Entry->load_iter( $terms, $args );

                undef $daily_list;
                while ( my $entry = $iter2->() ) {
                    ( $day_timestamp = $entry->authored_on ) =~ s/^(\d{4})(\d{2})(\d{2})\d{6}$/$1$2${3}000000/;
                    $daily_list->{ $day_timestamp } = 1;
                }
                @day_times = ();
                for my $key ( keys %$daily_list ) {
                    push @day_times, $key;
                }

                foreach my $time ( @day_times ) {
                    push @dauthors, $blog_id . ':' . $author->id . "#" . $time;
                }
                @dauthors = sort { $b cmp $a } @dauthors;
            }
            }
        }


        # index
        if ( $type eq 'index' ) {
            my $iter = MT::Template->load_iter( { blog_id => $blog_id, type => 'index', rebuild_me => [1, 2] } );
            while ( my $template = $iter->() ) {
                push @template_ids, $blog_id . ':' . $template->id;
            }
            @template_ids = sort { $b cmp $a } @template_ids;
        }
    }

    $param{indexes} = \@template_ids;
    $param{entries} = \@entry_ids;
    $param{pages} = \@page_ids;
    $param{categories} = \@cats;
    $param{ycategories} = \@ycats;
    $param{mcategories} = \@mcats;
    $param{wcategories} = \@wcats;
    $param{dcategories} = \@dcats;
    $param{authors} = \@authors;
    $param{yauthors} = \@yauthors;
    $param{mauthors} = \@mauthors;
    $param{wauthors} = \@wauthors;
    $param{dauthors} = \@dauthors;
    if ( @year_times ) {
        $param{yearly} = \@year_times;
    }
    if ( @month_times ) {
        $param{monthly} = \@month_times;
    }
    if ( @week_times ) {
        $param{weekly} = \@week_times;
    }
    if ( @day_times ) {
        $param{daily} = \@day_times;
    }

    my $jsonp = 'callback'; #$callback;
    $app->jsonp_result( \%param, $jsonp );
}

sub jsonp_result {
    my $app = shift;
    my ( $result, $jsonp ) = @_;
    $app->send_http_header("text/javascript+json");
    $app->{no_print_body} = 1;

    my $json = MT::Util::to_json($result);
    $app->print($json);
    return undef;
}

1;
